<?php
/*********************************************************************
 *
 * $Id: yocto_multiaxiscontroller.php 26674 2017-02-28 13:44:41Z seb $
 *
 * Implements YMultiAxisController, the high-level API for MultiAxisController functions
 *
 * - - - - - - - - - License information: - - - - - - - - - 
 *
 *  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
 *
 *  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
 *  non-exclusive license to use, modify, copy and integrate this
 *  file into your software for the sole purpose of interfacing
 *  with Yoctopuce products.
 *
 *  You may reproduce and distribute copies of this file in
 *  source or object form, as long as the sole purpose of this
 *  code is to interface with Yoctopuce products. You must retain
 *  this notice in the distributed source file.
 *
 *  You should refer to Yoctopuce General Terms and Conditions
 *  for additional information regarding your rights and
 *  obligations.
 *
 *  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
 *  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING 
 *  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
 *  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
 *  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
 *  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
 *  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
 *  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
 *  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
 *  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
 *  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
 *  WARRANTY, OR OTHERWISE.
 *
 *********************************************************************/

//--- (YMultiAxisController return codes)
//--- (end of YMultiAxisController return codes)
//--- (YMultiAxisController definitions)
if(!defined('Y_GLOBALSTATE_ABSENT'))         define('Y_GLOBALSTATE_ABSENT',        0);
if(!defined('Y_GLOBALSTATE_ALERT'))          define('Y_GLOBALSTATE_ALERT',         1);
if(!defined('Y_GLOBALSTATE_HI_Z'))           define('Y_GLOBALSTATE_HI_Z',          2);
if(!defined('Y_GLOBALSTATE_STOP'))           define('Y_GLOBALSTATE_STOP',          3);
if(!defined('Y_GLOBALSTATE_RUN'))            define('Y_GLOBALSTATE_RUN',           4);
if(!defined('Y_GLOBALSTATE_BATCH'))          define('Y_GLOBALSTATE_BATCH',         5);
if(!defined('Y_GLOBALSTATE_INVALID'))        define('Y_GLOBALSTATE_INVALID',       -1);
if(!defined('Y_NAXIS_INVALID'))              define('Y_NAXIS_INVALID',             YAPI_INVALID_UINT);
if(!defined('Y_COMMAND_INVALID'))            define('Y_COMMAND_INVALID',           YAPI_INVALID_STRING);
//--- (end of YMultiAxisController definitions)

//--- (YMultiAxisController declaration)
/**
 * YMultiAxisController Class: MultiAxisController function interface
 *
 * The Yoctopuce application programming interface allows you to drive a stepper motor.
 */
class YMultiAxisController extends YFunction
{
    const NAXIS_INVALID                  = YAPI_INVALID_UINT;
    const GLOBALSTATE_ABSENT             = 0;
    const GLOBALSTATE_ALERT              = 1;
    const GLOBALSTATE_HI_Z               = 2;
    const GLOBALSTATE_STOP               = 3;
    const GLOBALSTATE_RUN                = 4;
    const GLOBALSTATE_BATCH              = 5;
    const GLOBALSTATE_INVALID            = -1;
    const COMMAND_INVALID                = YAPI_INVALID_STRING;
    //--- (end of YMultiAxisController declaration)

    //--- (YMultiAxisController attributes)
    protected $_nAxis                    = Y_NAXIS_INVALID;              // UInt31
    protected $_globalState              = Y_GLOBALSTATE_INVALID;        // StepperState
    protected $_command                  = Y_COMMAND_INVALID;            // Text
    //--- (end of YMultiAxisController attributes)

    function __construct($str_func)
    {
        //--- (YMultiAxisController constructor)
        parent::__construct($str_func);
        $this->_className = 'MultiAxisController';

        //--- (end of YMultiAxisController constructor)
    }

    //--- (YMultiAxisController implementation)

    function _parseAttr($name, $val)
    {
        switch($name) {
        case 'nAxis':
            $this->_nAxis = intval($val);
            return 1;
        case 'globalState':
            $this->_globalState = intval($val);
            return 1;
        case 'command':
            $this->_command = $val;
            return 1;
        }
        return parent::_parseAttr($name, $val);
    }

    /**
     * Returns the number of synchronized controllers.
     *
     * @return an integer corresponding to the number of synchronized controllers
     *
     * On failure, throws an exception or returns Y_NAXIS_INVALID.
     */
    public function get_nAxis()
    {
        // $res                    is a int;
        if ($this->_cacheExpiration <= YAPI::GetTickCount()) {
            if ($this->load(YAPI::$defaultCacheValidity) != YAPI_SUCCESS) {
                return Y_NAXIS_INVALID;
            }
        }
        $res = $this->_nAxis;
        return $res;
    }

    /**
     * Changes the number of synchronized controllers.
     *
     * @param newval : an integer corresponding to the number of synchronized controllers
     *
     * @return YAPI_SUCCESS if the call succeeds.
     *
     * On failure, throws an exception or returns a negative error code.
     */
    public function set_nAxis($newval)
    {
        $rest_val = strval($newval);
        return $this->_setAttr("nAxis",$rest_val);
    }

    /**
     * Returns the stepper motor set overall state.
     *
     * @return a value among Y_GLOBALSTATE_ABSENT, Y_GLOBALSTATE_ALERT, Y_GLOBALSTATE_HI_Z,
     * Y_GLOBALSTATE_STOP, Y_GLOBALSTATE_RUN and Y_GLOBALSTATE_BATCH corresponding to the stepper motor
     * set overall state
     *
     * On failure, throws an exception or returns Y_GLOBALSTATE_INVALID.
     */
    public function get_globalState()
    {
        // $res                    is a enumSTEPPERSTATE;
        if ($this->_cacheExpiration <= YAPI::GetTickCount()) {
            if ($this->load(YAPI::$defaultCacheValidity) != YAPI_SUCCESS) {
                return Y_GLOBALSTATE_INVALID;
            }
        }
        $res = $this->_globalState;
        return $res;
    }

    public function get_command()
    {
        // $res                    is a string;
        if ($this->_cacheExpiration <= YAPI::GetTickCount()) {
            if ($this->load(YAPI::$defaultCacheValidity) != YAPI_SUCCESS) {
                return Y_COMMAND_INVALID;
            }
        }
        $res = $this->_command;
        return $res;
    }

    public function set_command($newval)
    {
        $rest_val = $newval;
        return $this->_setAttr("command",$rest_val);
    }

    /**
     * Retrieves a multi-axis controller for a given identifier.
     * The identifier can be specified using several formats:
     * <ul>
     * <li>FunctionLogicalName</li>
     * <li>ModuleSerialNumber.FunctionIdentifier</li>
     * <li>ModuleSerialNumber.FunctionLogicalName</li>
     * <li>ModuleLogicalName.FunctionIdentifier</li>
     * <li>ModuleLogicalName.FunctionLogicalName</li>
     * </ul>
     *
     * This function does not require that the multi-axis controller is online at the time
     * it is invoked. The returned object is nevertheless valid.
     * Use the method YMultiAxisController.isOnline() to test if the multi-axis controller is
     * indeed online at a given time. In case of ambiguity when looking for
     * a multi-axis controller by logical name, no error is notified: the first instance
     * found is returned. The search is performed first by hardware name,
     * then by logical name.
     *
     * @param func : a string that uniquely characterizes the multi-axis controller
     *
     * @return a YMultiAxisController object allowing you to drive the multi-axis controller.
     */
    public static function FindMultiAxisController($func)
    {
        // $obj                    is a YMultiAxisController;
        $obj = YFunction::_FindFromCache('MultiAxisController', $func);
        if ($obj == null) {
            $obj = new YMultiAxisController($func);
            YFunction::_AddToCache('MultiAxisController', $func, $obj);
        }
        return $obj;
    }

    public function sendCommand($command)
    {
        return $this->set_command($command);
    }

    /**
     * Reinitialize all controllers and clear all alert flags.
     *
     * @return YAPI_SUCCESS if the call succeeds.
     *         On failure, throws an exception or returns a negative error code.
     */
    public function reset()
    {
        return $this->sendCommand('Z');
    }

    /**
     * Starts all motors backward at the specified speeds, to search for the motor home position.
     *
     * @param speed : desired speed for all axis, in steps per second.
     *
     * @return YAPI_SUCCESS if the call succeeds.
     *         On failure, throws an exception or returns a negative error code.
     */
    public function findHomePosition($speed)
    {
        // $cmd                    is a str;
        // $i                      is a int;
        // $ndim                   is a int;
        $ndim = sizeof($speed);
        $cmd = sprintf('H%d', round(1000*$speed[0]));
        $i = 1;
        while ($i + 1 < $ndim) {
            $cmd = sprintf('%s,%d', $cmd, round(1000*$speed[$i]));
            $i = $i + 1;
        }
        return $this->sendCommand($cmd);
    }

    /**
     * Starts all motors synchronously to reach a given absolute position.
     * The time needed to reach the requested position will depend on the lowest
     * acceleration and max speed parameters configured for all motors.
     * The final position will be reached on all axis at the same time.
     *
     * @param absPos : absolute position, measured in steps from each origin.
     *
     * @return YAPI_SUCCESS if the call succeeds.
     *         On failure, throws an exception or returns a negative error code.
     */
    public function moveTo($absPos)
    {
        // $cmd                    is a str;
        // $i                      is a int;
        // $ndim                   is a int;
        $ndim = sizeof($absPos);
        $cmd = sprintf('M%d', round(16*$absPos[0]));
        $i = 1;
        while ($i + 1 < $ndim) {
            $cmd = sprintf('%s,%d', $cmd, round(16*$absPos[$i]));
            $i = $i + 1;
        }
        return $this->sendCommand($cmd);
    }

    /**
     * Starts all motors synchronously to reach a given relative position.
     * The time needed to reach the requested position will depend on the lowest
     * acceleration and max speed parameters configured for all motors.
     * The final position will be reached on all axis at the same time.
     *
     * @param relPos : relative position, measured in steps from the current position.
     *
     * @return YAPI_SUCCESS if the call succeeds.
     *         On failure, throws an exception or returns a negative error code.
     */
    public function moveRel($relPos)
    {
        // $cmd                    is a str;
        // $i                      is a int;
        // $ndim                   is a int;
        $ndim = sizeof($relPos);
        $cmd = sprintf('m%d', round(16*$relPos[0]));
        $i = 1;
        while ($i + 1 < $ndim) {
            $cmd = sprintf('%s,%d', $cmd, round(16*$relPos[$i]));
            $i = $i + 1;
        }
        return $this->sendCommand($cmd);
    }

    /**
     * Keep the motor in the same state for the specified amount of time, before processing next command.
     *
     * @param waitMs : wait time, specified in milliseconds.
     *
     * @return YAPI_SUCCESS if the call succeeds.
     *         On failure, throws an exception or returns a negative error code.
     */
    public function pause($waitMs)
    {
        return $this->sendCommand(sprintf('_%d',$waitMs));
    }

    /**
     * Stops the motor with an emergency alert, without taking any additional precaution.
     *
     * @return YAPI_SUCCESS if the call succeeds.
     *         On failure, throws an exception or returns a negative error code.
     */
    public function emergencyStop()
    {
        return $this->sendCommand('!');
    }

    /**
     * Stops the motor smoothly as soon as possible, without waiting for ongoing move completion.
     *
     * @return YAPI_SUCCESS if the call succeeds.
     *         On failure, throws an exception or returns a negative error code.
     */
    public function abortAndBrake()
    {
        return $this->sendCommand('B');
    }

    /**
     * Turn the controller into Hi-Z mode immediately, without waiting for ongoing move completion.
     *
     * @return YAPI_SUCCESS if the call succeeds.
     *         On failure, throws an exception or returns a negative error code.
     */
    public function abortAndHiZ()
    {
        return $this->sendCommand('z');
    }

    public function nAxis()
    { return $this->get_nAxis(); }

    public function setNAxis($newval)
    { return $this->set_nAxis($newval); }

    public function globalState()
    { return $this->get_globalState(); }

    public function command()
    { return $this->get_command(); }

    public function setCommand($newval)
    { return $this->set_command($newval); }

    /**
     * Continues the enumeration of multi-axis controllers started using yFirstMultiAxisController().
     *
     * @return a pointer to a YMultiAxisController object, corresponding to
     *         a multi-axis controller currently online, or a null pointer
     *         if there are no more multi-axis controllers to enumerate.
     */
    public function nextMultiAxisController()
    {   $resolve = YAPI::resolveFunction($this->_className, $this->_func);
        if($resolve->errorType != YAPI_SUCCESS) return null;
        $next_hwid = YAPI::getNextHardwareId($this->_className, $resolve->result);
        if($next_hwid == null) return null;
        return self::FindMultiAxisController($next_hwid);
    }

    /**
     * Starts the enumeration of multi-axis controllers currently accessible.
     * Use the method YMultiAxisController.nextMultiAxisController() to iterate on
     * next multi-axis controllers.
     *
     * @return a pointer to a YMultiAxisController object, corresponding to
     *         the first multi-axis controller currently online, or a null pointer
     *         if there are none.
     */
    public static function FirstMultiAxisController()
    {   $next_hwid = YAPI::getFirstHardwareId('MultiAxisController');
        if($next_hwid == null) return null;
        return self::FindMultiAxisController($next_hwid);
    }

    //--- (end of YMultiAxisController implementation)

};

//--- (MultiAxisController functions)

/**
 * Retrieves a multi-axis controller for a given identifier.
 * The identifier can be specified using several formats:
 * <ul>
 * <li>FunctionLogicalName</li>
 * <li>ModuleSerialNumber.FunctionIdentifier</li>
 * <li>ModuleSerialNumber.FunctionLogicalName</li>
 * <li>ModuleLogicalName.FunctionIdentifier</li>
 * <li>ModuleLogicalName.FunctionLogicalName</li>
 * </ul>
 *
 * This function does not require that the multi-axis controller is online at the time
 * it is invoked. The returned object is nevertheless valid.
 * Use the method YMultiAxisController.isOnline() to test if the multi-axis controller is
 * indeed online at a given time. In case of ambiguity when looking for
 * a multi-axis controller by logical name, no error is notified: the first instance
 * found is returned. The search is performed first by hardware name,
 * then by logical name.
 *
 * @param func : a string that uniquely characterizes the multi-axis controller
 *
 * @return a YMultiAxisController object allowing you to drive the multi-axis controller.
 */
function yFindMultiAxisController($func)
{
    return YMultiAxisController::FindMultiAxisController($func);
}

/**
 * Starts the enumeration of multi-axis controllers currently accessible.
 * Use the method YMultiAxisController.nextMultiAxisController() to iterate on
 * next multi-axis controllers.
 *
 * @return a pointer to a YMultiAxisController object, corresponding to
 *         the first multi-axis controller currently online, or a null pointer
 *         if there are none.
 */
function yFirstMultiAxisController()
{
    return YMultiAxisController::FirstMultiAxisController();
}

//--- (end of MultiAxisController functions)
?>